﻿--[[ Postal_Select: Adds checkboxes to the inbox with buttons for returning/opening. ]]--

assert( Postal, "Postal not found!")

------------------------------
--      Are you local?      --
------------------------------

local postallib = AceLibrary("PostalLib-1.0")
local L = AceLibrary("AceLocale-2.2"):new("Postal")
local _G = getfenv(0)

local indexAdjust = 0
local currentMode = nil
local toProcess = 1
local selectedMail = {}

local openButton = nil
local returnButton = nil

local checkboxFunc = function() Postal_Select:ToggleMail(this) end

-- based on a free function at wowwiki
local function tremovebyval(tab, val)
	for k,v in pairs(tab) do
		if v == val then
			table.remove(tab, k)
			return true
		end
	end
	return false
end

----------------------------------
--      Module Declaration      --
----------------------------------

Postal_Select = Postal:NewModule("Select")
Postal_Select.revision = tonumber(string.sub("$Revision: 26360 $", 12, -3))

function Postal_Select:OnEnable()
	--create the open button
	if not openButton then
		openButton = CreateFrame("Button", "PostalSelectOpenButton", InboxFrame, "UIPanelButtonTemplate")
		openButton:SetWidth(120)
		openButton:SetHeight(25)
		openButton:SetPoint("RIGHT", InboxFrame, "TOP", 5, -53)
		openButton:SetText(L["Open"])
		openButton:SetScript("OnClick", function() Postal_Select:HandleSelect(1) end)
	end

	--create the return button
	if not returnButton then
		returnButton = CreateFrame("Button", "PostalSelectReturnButton", InboxFrame, "UIPanelButtonTemplate")
		returnButton:SetWidth(120)
		returnButton:SetHeight(25)
		returnButton:SetPoint("LEFT", InboxFrame, "TOP", 10, -53)
		returnButton:SetText(L["Return"])
		returnButton:SetScript("OnClick", function() Postal_Select:HandleSelect() end)
	end

	--indent to make room for the checkboxes
	MailItem1:SetPoint("TOPLEFT", "InboxFrame", "TOPLEFT", 48, -80)
	for i = 1, 7 do
		_G["MailItem"..i.."ExpireTime"]:SetPoint("TOPRIGHT", "MailItem"..i, "TOPRIGHT", 10, -4)
		_G["MailItem"..i]:SetWidth(280)
	end

	--now create the checkboxes
	for i = 1, 7 do
		if not _G["PostalInboxCB"..i] then
			local CB = CreateFrame("CheckButton", "PostalInboxCB"..i, _G["MailItem"..i], "OptionsCheckButtonTemplate")
			CB:SetID(i)
			CB:SetPoint("RIGHT", "MailItem"..i, "LEFT", 1, 0)
			CB:SetWidth(24)
			CB:SetHeight(24)
			CB:SetHitRectInsets(0, 0, 0, 0)
			CB:SetScript("OnClick", checkboxFunc)
		end
	end

	--hook shizz
	self:Hook("InboxFrame_Update", true)
	--make sure to reset on mail close
	self:RegisterEvent("MAIL_CLOSE", "Reset")
end

function Postal_Select:ToggleMail(frame)
	local index = frame:GetID() + (InboxFrame.pageNum - 1) * 7
	if not frame:GetChecked() then
		tremovebyval(selectedMail, index)
	else
		table.insert(selectedMail, index)
	end
end

function Postal_Select:HandleSelect(mode)
	if not selectedMail[1] then return end
	currentMode = mode

	postallib:DisableInbox(1)

	toProcess = #selectedMail
	table.sort(selectedMail)
	indexAdjust = 0

	self:RegisterEvent("Postal10_MailPurged")
	self:RegisterEvent("Postal10_MailSkipped", "ProcessNext")

	if currentMode then
		postallib:PurgeMail(selectedMail[1])
	else
		postallib:ReturnMail(selectedMail[1])
	end
end

function Postal_Select:Postal10_MailPurged()
	indexAdjust = indexAdjust + 1
	self:ProcessNext()
end

function Postal_Select:ProcessNext()
	InboxFrame_Update()
	toProcess = toProcess - 1

	if toProcess > 0 then
		table.remove(selectedMail, 1)
		selectedMail[1] = selectedMail[1] - indexAdjust
		if currentMode then
			postallib:PurgeMail(selectedMail[1])
		else
			postallib:ReturnMail(selectedMail[1])
		end
	else
		self:Reset()
	end
end

function Postal_Select:InboxFrame_Update()
	self.hooks["InboxFrame_Update"]()
	for i = 1, 7 do
		local index = i + (InboxFrame.pageNum-1)*7
		local CB = _G["PostalInboxCB"..i]
		if index > GetInboxNumItems() then
			CB:Hide()
		else
			CB:Show()
			CB:SetChecked(nil)
			for key, value in pairs(selectedMail) do
				if value == index then
					CB:SetChecked(1)
					break
				end
			end
		end
	end
end

function Postal_Select:Reset()
	self:UnregisterEvent("Postal10_MailPurged")
	self:UnregisterEvent("Postal10_MailSkipped")

	for k in pairs(selectedMail) do
		selectedMail[k] = nil
	end
	toProcess = 1
	postallib:DisableInbox()
	InboxFrame_Update()
end

